from _typeshed import Incomplete
from typing import Any

ipaddress: Any

def register_ipaddress(conn_or_curs: Incomplete | None = ...) -> None: ...
def cast_interface(s, cur: Incomplete | None = ...): ...
def cast_network(s, cur: Incomplete | None = ...): ...
def adapt_ipaddress(obj): ...
